CREATE OR REPLACE PROCEDURE SP_ALTA_PRODUCTE (
	p_nom				IN PRODUCTE.nom%TYPE,
	p_formula			IN PRODUCTE.formula%TYPE,
	p_format			IN PRODUCTE.format%TYPE,
	p_presentacio		IN PRODUCTE.presentacio%TYPE,
	p_marca				IN PRODUCTE.marca%TYPE,
	p_pais				IN PRODUCTE.pais%TYPE,
	p_distribuidor		IN PRODUCTE.distribuidor%TYPE,
	RSP					OUT VARCHAR2
)
IS
/******************************************************************************
   Nom:                 SP_ALTA_PRODUCTE
   Propsit:            afegeix un nou producte a la taula PRODUCTE. Tamb serveix
						per iniciar els trmits del registre.
                   
   Parmetres entrada:  - p_nom VARCHAR2(50): nom del producte.
                        - p_formula VARCHAR2(20): frmula de fabricaci.
						- p_marca NUMBER: codi de la marca.
						- p_format NUMBER: codi del format.
						- p_presentacio NUMBER: codi de la presentaci.
						- p_pais NUMBER: identificador del pas.
						- p_distribuidor NUMBER: identificador del distribudor.						
						
   Parmetres sortida:  RSP
   Retorna:             'OK' si l'execuci ha finalitzat amb xit
                        'ERROR: tipus d'error' si ha fracassat
******************************************************************************/
paramsEntrada	VARCHAR2(500);
existeix		INTEGER;
paisDist		INTEGER;
codi			NUMBER;
err_msg			VARCHAR2(200);
camp_nul		VARCHAR2(20);
ref_err			VARCHAR2(20);
E_NUL			EXCEPTION;
E_NO_FK			EXCEPTION;
E_NO_DIST		EXCEPTION;

BEGIN
   
paramsEntrada:= 'p_nom: '||p_nom||
				', p_formula: '||p_formula||
				', p_format: '||p_format||
				', p_presentacio: '||p_presentacio||
				', p_marca: '||p_marca||				
				', p_pais: '||p_pais||
				', p_distribuidor: '||p_distribuidor;
   
--Verifica que els camps obligatoris no siguin nuls
IF (p_nom IS NULL) THEN 
	camp_nul:= 'nom';
	RAISE E_NUL;
ELSIF (p_formula IS NULL) THEN 
	camp_nul:= 'formula';
	RAISE E_NUL;
ELSIF (p_format IS NULL) THEN 
	camp_nul:= 'format';
	RAISE E_NUL;
ELSIF (p_presentacio IS NULL) THEN 
	camp_nul:= 'presentacio';
	RAISE E_NUL;
ELSIF (p_marca IS NULL) THEN 
	camp_nul:= 'marca';
	RAISE E_NUL;
ELSIF (p_pais IS NULL) THEN 
	camp_nul:= 'pais';
	RAISE E_NUL;
ELSIF (p_distribuidor IS NULL) THEN 
	camp_nul:= 'distribuidor';
	RAISE E_NUL;
END IF;

--Verifica que existeix el format
SELECT COUNT (*) INTO existeix
	FROM FORMAT
	WHERE idFormat = p_format;
	
	IF (existeix = 0) THEN
		ref_err:= 'format'; 
		RAISE E_NO_FK;
	END IF;
	
--Verifica que existeix la presentacio
SELECT COUNT (*) INTO existeix
	FROM PRESENTACIO
	WHERE idPresentacio = p_presentacio;
	
	IF (existeix = 0) THEN
		ref_err:= 'presentacio'; 
		RAISE E_NO_FK;
	END IF;

--Verifica que existeix la marca
SELECT COUNT (*) INTO existeix
	FROM MARCA
	WHERE idMarca = p_marca;
	
	IF (existeix = 0) THEN
		ref_err:= 'marca'; 
		RAISE E_NO_FK;
	END IF;

--Verifica que existeix el pas
SELECT COUNT (*) INTO existeix
	FROM PAIS
	WHERE idPais = p_pais;
	
	IF (existeix = 0) THEN
		ref_err:= 'pais'; 
		RAISE E_NO_FK;
	END IF;

--Verifica que existeix el distribudor 
SELECT COUNT (*) INTO existeix
	FROM DISTRIBUIDOR
	WHERE idDistribuidor = p_distribuidor;
	
	IF (existeix = 0) THEN
		ref_err:= 'distribuidor'; 
		RAISE E_NO_FK;
	END IF;
	
--Verifica que el distribudor s del pas corresponent
SELECT COUNT (*) INTO paisDist
	FROM DISTRIBUIDOR d, PAIS p
	WHERE d.pais = p_distribuidor AND p.idPais = p_pais;
	
	IF (paisDist = 0) THEN
		RAISE E_NO_DIST;
	END IF;

--Inserta el nou producte
INSERT INTO PRODUCTE (nom, formula, format, presentacio, marca, pais, distribuidor, registrat)
VALUES (p_nom, p_formula, p_format, p_presentacio, p_marca, p_pais, p_distribuidor, DEFAULT);

--Desem el idProducte assignat
SELECT idProducte INTO codi
	FROM PRODUCTE
	WHERE nom = p_nom AND formula = p_formula;
	
--Comencem els tramits pel registre ja que, amb cada nou producte, hi ha un nou proces
SP_ALTA_REGISTRE(codi, p_pais, p_distribuidor, null, SYSDATE, null, null, RSP);

--------------------------------------------------------------------------------------
-- 									ACTUALITZA EST6 								--
--------------------------------------------------------------------------------------

--Actualitza la taula EST6 un cop donat d'alta el producte
SP_MODIFICACIO_EST6(); 
--------------------------------------------------------------------------------------

-- Finalitza amb xit
RSP:= 'OK';
SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_PRODUCTE', paramsEntrada, 'RSP: '||RSP);

COMMIT;

EXCEPTION

	WHEN E_NUL THEN
		RSP:= 'ERROR: el camp ' || camp_nul || ' no pot ser nul'; 
		SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_PRODUCTE', paramsEntrada, 'RSP: '||RSP);
		
	WHEN E_NO_FK THEN
		RSP:= 'ERROR: ' || ref_err || ' no existeix';
		SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_PRODUCTE', paramsEntrada, 'RSP: '||RSP);

	WHEN E_NO_DIST THEN
		RSP:= 'ERROR: el distribudor no pertany al pas';
		SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_PRODUCTE', paramsEntrada, 'RSP: '||RSP);
	
	WHEN OTHERS THEN
		ROLLBACK;
	
		IF (SQLCODE = -00001) THEN
			RSP:= 'ERROR: ja existeix un producte amb el mateix: nom, frmula.';
			SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_PRODUCTE', paramsEntrada, 'RSP: '||RSP);
			
		ELSE
			err_msg := SUBSTR (SQLERRM, 1, 200);
			RSP:= 'ERROR: ' || err_msg;
			SP_ALTA_LOGS(to_char(sysdate,'DD/MM/YYYY HH24:MI:SS'), 'SP_ALTA_PRODUCTE', paramsEntrada, 'RSP: '||RSP);
		END IF;

END SP_ALTA_PRODUCTE;